searchNodes=[{"doc":"Specter is a method for managing data structures and entities provided by webrtc.rs . It is intended as a low-level library with some small set of opinions, which can composed into more complex behaviors by higher-level libraries and applications. Key points Specter wraps webrtc.rs , which heavily utilizes async Rust. For this reason, many functions cannot be automatically awaited by the caller—the NIF functions send messages across channels to separate threads managed by Rust, which send messages back to Elixir that can be caught by receive or handle_info . Usage A process initializes Specter via the init/1 function, which registers the current process for callbacks that may be triggered via webrtc entities. iex&gt; ## Initialize the library. Register messages to the current pid. iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) ...&gt; iex&gt; ## Create a peer connection&#39;s dependencies iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) ...&gt; iex&gt; Specter . media_engine_exists? ( specter , media_engine ) true iex&gt; Specter . registry_exists? ( specter , registry ) true ...&gt; iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) ...&gt; iex&gt; Specter . media_engine_exists? ( specter , media_engine ) false iex&gt; Specter . registry_exists? ( specter , registry ) false ...&gt; iex&gt; ## Create a peer connection iex&gt; { :ok , pc_1 } = Specter . new_peer_connection ( specter , api ) iex&gt; assert_receive { :peer_connection_ready , ^ pc_1 } iex&gt; Specter . peer_connection_exists? ( specter , pc_1 ) true iex&gt; ## Add a thing to be negotiated iex&gt; :ok = Specter . create_data_channel ( specter , pc_1 , &quot;data&quot; ) iex&gt; assert_receive { :data_channel_created , ^ pc_1 } ...&gt; iex&gt; ## Create an offer iex&gt; :ok = Specter . create_offer ( specter , pc_1 ) iex&gt; assert_receive { :offer , ^ pc_1 , offer } iex&gt; :ok = Specter . set_local_description ( specter , pc_1 , offer ) iex&gt; assert_receive { :ok , ^ pc_1 , :set_local_description } ...&gt; iex&gt; ## Create a second peer connection, to answer back iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc_2 } = Specter . new_peer_connection ( specter , api ) iex&gt; assert_receive { :peer_connection_ready , ^ pc_2 } ...&gt; iex&gt; ## Begin negotiating offer/answer iex&gt; :ok = Specter . set_remote_description ( specter , pc_2 , offer ) iex&gt; assert_receive { :ok , ^ pc_2 , :set_remote_description } iex&gt; :ok = Specter . create_answer ( specter , pc_2 ) iex&gt; assert_receive { :answer , ^ pc_2 , answer } iex&gt; :ok = Specter . set_local_description ( specter , pc_2 , answer ) iex&gt; assert_receive { :ok , ^ pc_2 , :set_local_description } ...&gt; iex&gt; ## Receive ice candidates iex&gt; assert_receive { :ice_candidate , ^ pc_1 , _candidate } iex&gt; assert_receive { :ice_candidate , ^ pc_2 , _candidate } ...&gt; iex&gt; ## Shut everything down iex&gt; Specter . close_peer_connection ( specter , pc_1 ) :ok iex&gt; assert_receive { :peer_connection_closed , ^ pc_1 } ...&gt; iex&gt; :ok = Specter . close_peer_connection ( specter , pc_2 ) iex&gt; assert_receive { :peer_connection_closed , ^ pc_2 } Thoughts During development of the library, it can be assumed that callers will implement handle_info/2 function heads appropriate to the underlying implementation. Once these are more solid, it would be nice to use Specter , which will inject a handle_info/2 callback, and send the messages to other callback functions defined by a behaviour. handle_ice_candidate , and so on. Some things are returned from the NIF as UUIDs. These are declared as @opaque , to indicate that users of the library should not rely of them being in a particular format. They could change later to be references, for instance.","ref":"Specter.html","title":"Specter","type":"module"},{"doc":"Given an ICE candidate, add it to the given peer connection. Assumes trickle ICE. Candidates must be JSON, with the keys candidate , sdp_mid , sdp_mline_index , and username_fragment .","ref":"Specter.html#add_ice_candidate/3","title":"Specter.add_ice_candidate/3","type":"function"},{"doc":"Closes an open instance of an RTCPeerConnection. Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc } = Specter . new_peer_connection ( specter , api ) iex&gt; assert_receive { :peer_connection_ready , ^ pc } ...&gt; iex&gt; Specter . close_peer_connection ( specter , pc ) :ok iex&gt; { :ok , _pc } = ...&gt; receive do ...&gt; { :peer_connection_closed , ^ pc } -&gt; { :ok , pc } ...&gt; after ...&gt; 500 -&gt; { :error , :timeout } ...&gt; end ...&gt; iex&gt; Specter . peer_connection_exists? ( specter , pc ) false","ref":"Specter.html#close_peer_connection/2","title":"Specter.close_peer_connection/2","type":"function"},{"doc":"Returns the current configuration for the initialized NIF. Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.example.com:3478&quot; ] ) iex&gt; Specter . config ( specter ) { :ok , % Specter.Config { ice_servers : [ &quot;stun:stun.example.com:3478&quot; ] } }","ref":"Specter.html#config/1","title":"Specter.config/1","type":"function"},{"doc":"Given an RTCPeerConnection where the remote description has been assigned via set_remote_description/4 , create an answer that can be passed to another connection. param type default specter t() peer_connection opaque options answer_options_t() voice_activity_detection: false","ref":"Specter.html#create_answer/3","title":"Specter.create_answer/3","type":"function"},{"doc":"Creates a data channel on an RTCPeerConnection. Note: this can be useful when attempting to generate a valid offer, but where no media tracks are expected to be sent or received. Callbacks from data channels have not yet been implemented.","ref":"Specter.html#create_data_channel/3","title":"Specter.create_data_channel/3","type":"function"},{"doc":"Given an RTCPeerConnection, create an offer that can be passed to another connection. param type default specter t() peer_connection opaque options offer_options_t() voice_activity_detection: false ice_restart: false","ref":"Specter.html#create_offer/3","title":"Specter.create_offer/3","type":"function"},{"doc":"Sends back the value of the current session description on a peer connection. This will send back JSON representing an offer or an answer when the peer connection has had set_local_description/3 called and has successfully negotiated ICE. In all other cases, nil will be sent. See pending_local_description/2 and local_description/2 . Usage iex&gt; { :ok , specter } = Specter . init ( ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc } = Specter . new_peer_connection ( specter , api ) iex&gt; assert_receive { :peer_connection_ready , ^ pc } iex&gt; Specter . current_local_description ( specter , pc ) :ok iex&gt; assert_receive { :current_local_description , ^ pc , nil }","ref":"Specter.html#current_local_description/2","title":"Specter.current_local_description/2","type":"function"},{"doc":"Initialize the library. This registers the calling process to receive callback messages to handle_info/2 . param type default ice_servers list(String.t()) [&quot;stun:stun.l.google.com:19302&quot;] Usage iex&gt; { :ok , _specter } = Specter . init ( ice_servers : [ &quot;stun:stun.example.com:3478&quot; ] )","ref":"Specter.html#init/1","title":"Specter.init/1","type":"function"},{"doc":"Sends back the value of the local session description on a peer connection. This will send back JSON representing an offer or an answer when the peer connection has had set_local_description/3 called. If ICE has been successfully negotated, the current local description will be sent back, otherwise the caller will receive the pending local description. See current_local_description/2 and pending_local_description/2 . Usage iex&gt; { :ok , specter } = Specter . init ( ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc } = Specter . new_peer_connection ( specter , api ) iex&gt; assert_receive { :peer_connection_ready , ^ pc } ...&gt; iex&gt; Specter . local_description ( specter , pc ) :ok iex&gt; assert_receive { :local_description , ^ pc , nil } ...&gt; iex&gt; :ok = Specter . create_offer ( specter , pc ) iex&gt; assert_receive { :offer , ^ pc , offer } iex&gt; :ok = Specter . set_local_description ( specter , pc , offer ) iex&gt; assert_receive { :ok , ^ pc , :set_local_description } ...&gt; iex&gt; Specter . local_description ( specter , pc ) :ok iex&gt; assert_receive { :local_description , ^ pc , ^ offer }","ref":"Specter.html#local_description/2","title":"Specter.local_description/2","type":"function"},{"doc":"Returns true or false, depending on whether the media engine is available for consumption, i.e. is initialized and has not been used by a function that takes ownership of it. Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; Specter . media_engine_exists? ( specter , media_engine ) true iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; Specter . media_engine_exists? ( specter , UUID . uuid4 ( ) ) false","ref":"Specter.html#media_engine_exists?/2","title":"Specter.media_engine_exists?/2","type":"function"},{"doc":"An APIBuilder is used to create RTCPeerConnections. This accepts as parameters the output of init/1 , new_media_enine/1 , and new_registry/2 . Note that this takes ownership of both the media engine and the registry, effectively consuming them. param type default specter t() media_engine opaque registry opaque Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , _api } = Specter . new_api ( specter , media_engine , registry )","ref":"Specter.html#new_api/3","title":"Specter.new_api/3","type":"function"},{"doc":"Creates a MediaEngine to be configured and used by later function calls. Codecs and other high level configuration are done on instances of MediaEngines. A MediaEngine is combined with a Registry in an entity called an APIBuilder, which is then used to create RTCPeerConnections. Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , _media_engine } = Specter . new_media_engine ( specter )","ref":"Specter.html#new_media_engine/1","title":"Specter.new_media_engine/1","type":"function"},{"doc":"Creates a new RTCPeerConnection, using an API reference created with new_api/3 . The functionality wrapped by this function is async, so :ok is returned immediately. Callers should listen for the {:peer_connection_ready, peer_connection_t()} message to receive the results of this function. param type default specter t() api opaque Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc } = Specter . new_peer_connection ( specter , api ) ...&gt; iex&gt; { :ok , _pc } = ...&gt; receive do ...&gt; { :peer_connection_ready , ^ pc } -&gt; { :ok , pc } ...&gt; after ...&gt; 500 -&gt; { :error , :timeout } ...&gt; end","ref":"Specter.html#new_peer_connection/2","title":"Specter.new_peer_connection/2","type":"function"},{"doc":"Creates an intercepter registry. This is a user configurable RTP/RTCP pipeline, and provides features such as NACKs and RTCP Reports. A registry must be created for each peer connection. The registry may be combined with a MediaEngine in an API (consuming both). The API instance is then used to create RTCPeerConnections. Note that creating a registry does not take ownership of the media engine. Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , _registry } = Specter . new_registry ( specter , media_engine ) ...&gt; iex&gt; Specter . media_engine_exists? ( specter , media_engine ) true","ref":"Specter.html#new_registry/2","title":"Specter.new_registry/2","type":"function"},{"doc":"Returns true or false, depending on whether the RTCPeerConnection is initialized. Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc } = Specter . new_peer_connection ( specter , api ) iex&gt; assert_receive { :peer_connection_ready , ^ pc } iex&gt; Specter . peer_connection_exists? ( specter , pc ) true iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; Specter . peer_connection_exists? ( specter , UUID . uuid4 ( ) ) false","ref":"Specter.html#peer_connection_exists?/2","title":"Specter.peer_connection_exists?/2","type":"function"},{"doc":"Sends back the value of the session description on a peer connection that is pending connection, or nil. Usage iex&gt; { :ok , specter } = Specter . init ( ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc } = Specter . new_peer_connection ( specter , api ) iex&gt; assert_receive { :peer_connection_ready , ^ pc } ...&gt; iex&gt; Specter . pending_local_description ( specter , pc ) :ok iex&gt; assert_receive { :pending_local_description , ^ pc , nil } ...&gt; iex&gt; :ok = Specter . create_offer ( specter , pc ) iex&gt; assert_receive { :offer , ^ pc , offer } iex&gt; :ok = Specter . set_local_description ( specter , pc , offer ) iex&gt; assert_receive { :ok , ^ pc , :set_local_description } ...&gt; iex&gt; Specter . pending_local_description ( specter , pc ) :ok iex&gt; assert_receive { :pending_local_description , ^ pc , ^ offer }","ref":"Specter.html#pending_local_description/2","title":"Specter.pending_local_description/2","type":"function"},{"doc":"Returns true or false, depending on whether the registry is available for consumption, i.e. is initialized and has not been used by a function that takes ownership of it. Usage iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; Specter . registry_exists? ( specter , registry ) true iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) iex&gt; Specter . registry_exists? ( specter , UUID . uuid4 ( ) ) false","ref":"Specter.html#registry_exists?/2","title":"Specter.registry_exists?/2","type":"function"},{"doc":"Given an offer or an answer session description, sets the local description on a peer connection. The description should be in the form of JSON with the keys type and sdp . param type default specter t/0 peer_connection opaque description t:session_description_t()","ref":"Specter.html#set_local_description/3","title":"Specter.set_local_description/3","type":"function"},{"doc":"Given an offer or an answer in the form of SDP generated by a remote party, sets the remote description on a peer connection. Expects a session description in the form of JSON with the keys type and sdp . param type default specter t/0 peer_connection opaque description session_description_t/0","ref":"Specter.html#set_remote_description/3","title":"Specter.set_remote_description/3","type":"function"},{"doc":"Options for creating a webrtc answer. Values default to false.","ref":"Specter.html#t:answer_options_t/0","title":"Specter.answer_options_t/0","type":"type"},{"doc":"Specter.api_t/0 represent an instantiated API managed in the NIF.","ref":"Specter.html#t:api_t/0","title":"Specter.api_t/0","type":"opaque"},{"doc":"An ICE candidate as JSON.","ref":"Specter.html#t:ice_candidate_t/0","title":"Specter.ice_candidate_t/0","type":"type"},{"doc":"A uri in the form protocol:host:port , where protocol is either stun or turn . Defaults to stun:stun.l.google.com:19302 .","ref":"Specter.html#t:ice_server/0","title":"Specter.ice_server/0","type":"type"},{"doc":"Options for initializing RTCPeerConnections. This is set during initialization of the library, and later used when creating new connections.","ref":"Specter.html#t:init_options/0","title":"Specter.init_options/0","type":"type"},{"doc":"Specter.media_engine_t/0 represents an instantiated MediaEngine managed in the NIF.","ref":"Specter.html#t:media_engine_t/0","title":"Specter.media_engine_t/0","type":"opaque"},{"doc":"native_t/0 references are returned from the NIF, and represent state held in Rust code.","ref":"Specter.html#t:native_t/0","title":"Specter.native_t/0","type":"opaque"},{"doc":"Options for creating a webrtc offer. Values default to false.","ref":"Specter.html#t:offer_options_t/0","title":"Specter.offer_options_t/0","type":"type"},{"doc":"Specter.peer_connection_t/0 represents an instantiated RTCPeerConnection managed in the NIF.","ref":"Specter.html#t:peer_connection_t/0","title":"Specter.peer_connection_t/0","type":"opaque"},{"doc":"Specter.registry_t/0 represent an instantiated intercepter Registry managed in the NIF.","ref":"Specter.html#t:registry_t/0","title":"Specter.registry_t/0","type":"opaque"},{"doc":"A UTF-8 encoded string encapsulating either an offer or an answer.","ref":"Specter.html#t:sdp_t/0","title":"Specter.sdp_t/0","type":"type"},{"doc":"The type of an SDP message, either an :offer or an :answer .","ref":"Specter.html#t:sdp_type_t/0","title":"Specter.sdp_type_t/0","type":"type"},{"doc":"A UTF-8 encoded string encapsulating an Offer or an Answer in JSON. The keys are as follows: key type type offer , answer sdp `sdp_t()","ref":"Specter.html#t:session_description_t/0","title":"Specter.session_description_t/0","type":"type"},{"doc":"Specter.t/0 wraps the reference returned from init/1 . All functions interacting with NIF state take a Specter.t/0 as their first argument.","ref":"Specter.html#t:t/0","title":"Specter.t/0","type":"type"},{"doc":"A representation of configuration kept in the initialized NIF.","ref":"Specter.Config.html","title":"Specter.Config","type":"module"},{"doc":"A representation of the configuration kept by the initialized NIF. Note that this does not map 1:1 to actual webrtc.rs data structures, but is an Elixir data structure into which the NIF can encode its config.","ref":"Specter.Config.html#t:t/0","title":"Specter.Config.t/0","type":"type"},{"doc":"Initialize the library. Register messages to the current pid. Specter.init/1 iex&gt; { :ok , specter } = Specter . init ( ice_servers : [ &quot;stun:stun.l.google.com:19302&quot; ] ) Create a peer connection's dependencies. Creating an API consumes its MediaEngine and Registry, making them unavailable for future API instances. Specter.new_media_engine/1 Specter.new_registry/2 Specter.new_api/3 Specter.media_engine_exists?/2 Specter.registry_exists?/2 iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; true = Specter . media_engine_exists? ( specter , media_engine ) iex&gt; true = Specter . registry_exists? ( specter , registry ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; false = Specter . media_engine_exists? ( specter , media_engine ) iex&gt; false = Specter . registry_exists? ( specter , registry ) Create a peer connection Specter.new_peer_connection/2 Specter.peer_connection_exists?/2 iex&gt; { :ok , pc_1 } = Specter . new_peer_connection ( specter , api ) iex&gt; :ok = receive do : ( { :peer_connection_ready , ^ pc_1 } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) iex&gt; true = Specter . peer_connection_exists? ( specter , pc_1 ) Add a thing to be negotiated Specter.create_data_channel/3 iex&gt; :ok = Specter . create_data_channel ( specter , pc_1 , &quot;data&quot; ) iex&gt; :ok = receive do : ( { :data_channel_created , ^ pc_1 } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) Create an offer Specter.create_offer/2 Specter.set_local_description/3 iex&gt; :ok = Specter . create_offer ( specter , pc_1 ) iex&gt; { :ok , offer } = receive do : ( { :offer , ^ pc_1 , offer } -&gt; { :ok , offer } ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) iex&gt; :ok = Specter . set_local_description ( specter , pc_1 , offer ) iex&gt; :ok = receive do : ( { :ok , ^ pc_1 , :set_local_description } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) Create a second peer connection, to answer back iex&gt; { :ok , media_engine } = Specter . new_media_engine ( specter ) iex&gt; { :ok , registry } = Specter . new_registry ( specter , media_engine ) iex&gt; { :ok , api } = Specter . new_api ( specter , media_engine , registry ) iex&gt; { :ok , pc_2 } = Specter . new_peer_connection ( specter , api ) iex&gt; :ok = receive do : ( { :peer_connection_ready , ^ pc_2 } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) Begin negotiating offer/answer Specter.set_remote_description/3 Specter.create_answer/2 Specter.set_local_description/3 ## give the offer to the second peer connection iex&gt; :ok = Specter . set_remote_description ( specter , pc_2 , offer ) iex&gt; :ok = receive do : ( { :ok , ^ pc_2 , :set_remote_description } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) ## create an answer iex&gt; :ok = Specter . create_answer ( specter , pc_2 ) iex&gt; { :ok , answer } = receive do : ( { :answer , ^ pc_2 , answer } -&gt; { :ok , answer } ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) iex&gt; :ok = Specter . set_local_description ( specter , pc_2 , answer ) iex&gt; :ok = receive do : ( { :ok , ^ pc_2 , :set_local_description } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) ## give the answer to the first peer connection iex&gt; :ok = Specter . set_remote_description ( specter , pc_1 , answer ) iex&gt; :ok = receive do : ( { :ok , ^ pc_1 , :set_remote_description } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) Receive ice candidates Specter.add_ice_candidate/3 iex&gt; { :ok , candidate } = receive do : ( { :ice_candidate , ^ pc_1 , c } -&gt; { :ok , c } ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) iex&gt; :ok = Specter . add_ice_candidate ( specter , pc_2 , candidate ) ## .... and so on. iex&gt; { :ok , candidate } = receive do : ( { :ice_candidate , ^ pc_2 , c } -&gt; { :ok , c } ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) iex&gt; :ok = Specter . add_ice_candidate ( specter , pc_1 , candidate ) ## .... and so on. Shut everything down Specter.close_peer_connection/2 iex&gt; :ok = Specter . close_peer_connection ( specter , pc_1 ) iex&gt; receive do : ( { :peer_connection_closed , ^ pc_1 } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) :ok iex&gt; :ok = Specter . close_peer_connection ( specter , pc_2 ) iex&gt; receive do : ( { :peer_connection_closed , ^ pc_2 } -&gt; :ok ) , ...&gt; after : ( 100 -&gt; { :error , :timeout } ) :ok","ref":"lifecycle.html","title":"Lifecycle","type":"extras"},{"doc":"Specter is a wrapper for webrtc.rs as an Elixir NIF, using Rustler. This library is a low-level interface to the data structures and entities provided by Rust, with a minimal set of opinions.","ref":"readme.html","title":"Specter","type":"extras"},{"doc":"def deps do [ { :specter , &quot;~&gt; 0.1.0&quot; } ] end","ref":"readme.html#installation","title":"Specter - Installation","type":"extras"},{"doc":"[x] Specter.init/1 takes (opts) opts: ( ice_servers ) [x] Specter.config/1 (ref), returning Specter.Config.t() [x] Specter.new_media_engine/1 (ref), returning UUID [x] Specter.new_registry/2 (ref, uuid), returning UUID [x] Specter.new_api/3 (ref, uuid, uuid), returning UUID arg1: media engine uuid arg2: registry uuid [x] Specter.new_peer_connection/2 (ref, uuid), returning UUID arg1: api builder uuid [x] Specter.set_remote_description/3 (ref, uuid, json) [x] Specter.create_offer/3 (ref, uuid, opts) opts: ( voice_activity_detection : bool , ice_restart : bool ) [x] Specter.create_data_channel/3 (ref, uuid, label) [x] Specter.create_answer/3 (ref, uuid, opts) opts: ( voice_activity_detection : bool ) [x] Specter.set_local_description/3 (ref, uuid, json) [x] Specter.current_local_description/2 [x] Specter.pending_local_description/2 [x] Specter.local_description/2 [x] pc.on_ice_candidate sends candidate to callback process [x] Specter.add_ice_candidate/3 (ref, uuid, string) [ ] Specter.subscribe_to_state_changes (ref, uuid) state changes sent to Elixir pid [ ] pc.gathering_complete_promise sends message to callback process [ ] Specter.close (ref, uuid)","ref":"readme.html#checklist","title":"Specter - Checklist","type":"extras"},{"doc":"Development of Specter depends on Elixir, Erlang, and Rust being available in the environment. Suggested setup: asdf plugin-add erlang asdf plugin-add elixir asdf plugin-add rust bin/dev/doctor CI will run tests and audit the repository, but to make sure all checks locally, the following commands can be run: bin/dev/audit bin/dev/test The following script is encouraged to run all checks as a part of pushing commits: bin/dev/shipit","ref":"readme.html#development","title":"Specter - Development","type":"extras"},{"doc":"This project is indebted to the following sites and references (at the very least). https://github.com/scrogson/franz https://webrtc.rs","ref":"readme.html#references-thank-yous","title":"Specter - References / Thank yous","type":"extras"}]